# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "ResponseCreateParams",
    "InputUnionMember1",
    "InputUnionMember1EasyInputMessageParam",
    "InputUnionMember1EasyInputMessageParamContentUnionMember1",
    "InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputTextParam",
    "InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputImageParam",
    "InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputFileParam",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMessage",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContent",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputTextParam",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputImageParam",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputFileParam",
    "InputUnionMember1ResponseOutputMessageParam",
    "InputUnionMember1ResponseOutputMessageParamContent",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParam",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotation",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationFileCitation",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationURLCitation",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationContainerFileCitation",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationFilePath",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamLogprob",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamLogprobTopLogprob",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputRefusalParam",
    "InputUnionMember1ResponseFileSearchToolCallParam",
    "InputUnionMember1ResponseFileSearchToolCallParamResult",
    "InputUnionMember1ResponseComputerToolCallParam",
    "InputUnionMember1ResponseComputerToolCallParamAction",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionClick",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDoubleClick",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDrag",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDragPath",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionKeypress",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionMove",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionScreenshot",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionScroll",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionType",
    "InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionWait",
    "InputUnionMember1ResponseComputerToolCallParamPendingSafetyCheck",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutput",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutputOutput",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutputAcknowledgedSafetyCheck",
    "InputUnionMember1ResponseFunctionWebSearchParam",
    "InputUnionMember1ResponseFunctionWebSearchParamAction",
    "InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionSearch",
    "InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionOpenPage",
    "InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionFind",
    "InputUnionMember1ResponseFunctionToolCallParam",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamFunctionCallOutput",
    "InputUnionMember1ResponseReasoningItemParam",
    "InputUnionMember1ResponseReasoningItemParamSummary",
    "InputUnionMember1ResponseReasoningItemParamContent",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamImageGenerationCall",
    "InputUnionMember1ResponseCodeInterpreterToolCallParam",
    "InputUnionMember1ResponseCodeInterpreterToolCallParamOutput",
    "InputUnionMember1ResponseCodeInterpreterToolCallParamOutputOpenAITypesResponsesResponseCodeInterpreterToolCallParamOutputLogs",
    "InputUnionMember1ResponseCodeInterpreterToolCallParamOutputOpenAITypesResponsesResponseCodeInterpreterToolCallParamOutputImage",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCall",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCallAction",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCallOutput",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMcpListTools",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMcpListToolsTool",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMcpApprovalRequest",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMcpApprovalResponse",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamMcpCall",
    "InputUnionMember1ResponseCustomToolCallOutputParam",
    "InputUnionMember1ResponseCustomToolCallParam",
    "InputUnionMember1OpenAITypesResponsesResponseInputParamItemReference",
]


class ResponseCreateParams(TypedDict, total=False):
    input: Required[Union[str, Iterable[InputUnionMember1]]]

    model: Required[str]
    """model specified as `model_vendor/model`, for example `openai/gpt-4o`"""

    include: SequenceNotStr[str]
    """Which fields to include in the response"""

    instructions: str
    """Instructions for the response generation"""

    max_output_tokens: int
    """Maximum number of output tokens"""

    metadata: Dict[str, object]
    """Metadata for the response"""

    parallel_tool_calls: bool
    """Whether to enable parallel tool calls"""

    previous_response_id: str
    """ID of the previous response for chaining"""

    reasoning: Dict[str, object]
    """Reasoning configuration for the response"""

    store: bool
    """Whether to store the response"""

    stream: bool
    """Whether to stream the response"""

    temperature: float
    """Sampling temperature for randomness control"""

    text: Dict[str, object]
    """Text configuration parameters"""

    tool_choice: Union[str, Dict[str, object]]
    """Tool choice configuration"""

    tools: Iterable[Dict[str, object]]
    """Tools available for the response"""

    top_p: float
    """Top-p sampling parameter"""

    truncation: Literal["auto", "disabled"]
    """Truncation configuration"""


class InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputTextParam(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["input_text"]]


class InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputImageParam(TypedDict, total=False):
    detail: Required[Literal["low", "high", "auto"]]

    type: Required[Literal["input_image"]]

    file_id: str

    image_url: str


class InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputFileParam(TypedDict, total=False):
    type: Required[Literal["input_file"]]

    file_data: str

    file_id: str

    file_url: str

    filename: str


InputUnionMember1EasyInputMessageParamContentUnionMember1: TypeAlias = Union[
    InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputTextParam,
    InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputImageParam,
    InputUnionMember1EasyInputMessageParamContentUnionMember1ResponseInputFileParam,
]


class InputUnionMember1EasyInputMessageParam(TypedDict, total=False):
    content: Required[Union[str, Iterable[InputUnionMember1EasyInputMessageParamContentUnionMember1]]]

    role: Required[Literal["user", "assistant", "system", "developer"]]

    type: Literal["message"]


class InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputTextParam(
    TypedDict, total=False
):
    text: Required[str]

    type: Required[Literal["input_text"]]


class InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputImageParam(
    TypedDict, total=False
):
    detail: Required[Literal["low", "high", "auto"]]

    type: Required[Literal["input_image"]]

    file_id: str

    image_url: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputFileParam(
    TypedDict, total=False
):
    type: Required[Literal["input_file"]]

    file_data: str

    file_id: str

    file_url: str

    filename: str


InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContent: TypeAlias = Union[
    InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputTextParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputImageParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContentResponseInputFileParam,
]


class InputUnionMember1OpenAITypesResponsesResponseInputParamMessage(TypedDict, total=False):
    content: Required[Iterable[InputUnionMember1OpenAITypesResponsesResponseInputParamMessageContent]]

    role: Required[Literal["user", "system", "developer"]]

    status: Literal["in_progress", "completed", "incomplete"]

    type: Literal["message"]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationFileCitation(
    TypedDict, total=False
):
    file_id: Required[str]

    filename: Required[str]

    index: Required[int]

    type: Required[Literal["file_citation"]]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationURLCitation(
    TypedDict, total=False
):
    end_index: Required[int]

    start_index: Required[int]

    title: Required[str]

    type: Required[Literal["url_citation"]]

    url: Required[str]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationContainerFileCitation(
    TypedDict, total=False
):
    container_id: Required[str]

    end_index: Required[int]

    file_id: Required[str]

    filename: Required[str]

    start_index: Required[int]

    type: Required[Literal["container_file_citation"]]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationFilePath(
    TypedDict, total=False
):
    file_id: Required[str]

    index: Required[int]

    type: Required[Literal["file_path"]]


InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotation: TypeAlias = Union[
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationFileCitation,
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationURLCitation,
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationContainerFileCitation,
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotationOpenAITypesResponsesResponseOutputTextParamAnnotationFilePath,
]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamLogprobTopLogprob(
    TypedDict, total=False
):
    token: Required[str]

    bytes: Required[Iterable[int]]

    logprob: Required[float]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamLogprob(TypedDict, total=False):
    token: Required[str]

    bytes: Required[Iterable[int]]

    logprob: Required[float]

    top_logprobs: Required[
        Iterable[InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamLogprobTopLogprob]
    ]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParam(TypedDict, total=False):
    annotations: Required[Iterable[InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotation]]

    text: Required[str]

    type: Required[Literal["output_text"]]

    logprobs: Iterable[InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamLogprob]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputRefusalParam(TypedDict, total=False):
    refusal: Required[str]

    type: Required[Literal["refusal"]]


InputUnionMember1ResponseOutputMessageParamContent: TypeAlias = Union[
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParam,
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputRefusalParam,
]


class InputUnionMember1ResponseOutputMessageParam(TypedDict, total=False):
    id: Required[str]

    content: Required[Iterable[InputUnionMember1ResponseOutputMessageParamContent]]

    role: Required[Literal["assistant"]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["message"]]


class InputUnionMember1ResponseFileSearchToolCallParamResult(TypedDict, total=False):
    attributes: Dict[str, Union[str, float, bool]]

    file_id: str

    filename: str

    score: float

    text: str


class InputUnionMember1ResponseFileSearchToolCallParam(TypedDict, total=False):
    id: Required[str]

    queries: Required[SequenceNotStr[str]]

    status: Required[Literal["in_progress", "searching", "completed", "incomplete", "failed"]]

    type: Required[Literal["file_search_call"]]

    results: Iterable[InputUnionMember1ResponseFileSearchToolCallParamResult]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionClick(
    TypedDict, total=False
):
    button: Required[Literal["left", "right", "wheel", "back", "forward"]]

    type: Required[Literal["click"]]

    x: Required[int]

    y: Required[int]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDoubleClick(
    TypedDict, total=False
):
    type: Required[Literal["double_click"]]

    x: Required[int]

    y: Required[int]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDragPath(
    TypedDict, total=False
):
    x: Required[int]

    y: Required[int]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDrag(
    TypedDict, total=False
):
    path: Required[
        Iterable[
            InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDragPath
        ]
    ]

    type: Required[Literal["drag"]]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionKeypress(
    TypedDict, total=False
):
    keys: Required[SequenceNotStr[str]]

    type: Required[Literal["keypress"]]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionMove(
    TypedDict, total=False
):
    type: Required[Literal["move"]]

    x: Required[int]

    y: Required[int]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionScreenshot(
    TypedDict, total=False
):
    type: Required[Literal["screenshot"]]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionScroll(
    TypedDict, total=False
):
    scroll_x: Required[int]

    scroll_y: Required[int]

    type: Required[Literal["scroll"]]

    x: Required[int]

    y: Required[int]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionType(
    TypedDict, total=False
):
    text: Required[str]

    type: Required[Literal["type"]]


class InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionWait(
    TypedDict, total=False
):
    type: Required[Literal["wait"]]


InputUnionMember1ResponseComputerToolCallParamAction: TypeAlias = Union[
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionClick,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDoubleClick,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionDrag,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionKeypress,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionMove,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionScreenshot,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionScroll,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionType,
    InputUnionMember1ResponseComputerToolCallParamActionOpenAITypesResponsesResponseComputerToolCallParamActionWait,
]


class InputUnionMember1ResponseComputerToolCallParamPendingSafetyCheck(TypedDict, total=False):
    id: Required[str]

    code: Required[str]

    message: Required[str]


class InputUnionMember1ResponseComputerToolCallParam(TypedDict, total=False):
    id: Required[str]

    action: Required[InputUnionMember1ResponseComputerToolCallParamAction]

    call_id: Required[str]

    pending_safety_checks: Required[Iterable[InputUnionMember1ResponseComputerToolCallParamPendingSafetyCheck]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["computer_call"]]


class InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutputOutput(TypedDict, total=False):
    type: Required[Literal["computer_screenshot"]]

    file_id: str

    image_url: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutputAcknowledgedSafetyCheck(
    TypedDict, total=False
):
    id: Required[str]

    code: str

    message: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutput(TypedDict, total=False):
    call_id: Required[str]

    output: Required[InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutputOutput]

    type: Required[Literal["computer_call_output"]]

    id: str

    acknowledged_safety_checks: Iterable[
        InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutputAcknowledgedSafetyCheck
    ]

    status: Literal["in_progress", "completed", "incomplete"]


class InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionSearch(
    TypedDict, total=False
):
    query: Required[str]

    type: Required[Literal["search"]]


class InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionOpenPage(
    TypedDict, total=False
):
    type: Required[Literal["open_page"]]

    url: Required[str]


class InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionFind(
    TypedDict, total=False
):
    pattern: Required[str]

    type: Required[Literal["find"]]

    url: Required[str]


InputUnionMember1ResponseFunctionWebSearchParamAction: TypeAlias = Union[
    InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionSearch,
    InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionOpenPage,
    InputUnionMember1ResponseFunctionWebSearchParamActionOpenAITypesResponsesResponseFunctionWebSearchParamActionFind,
]


class InputUnionMember1ResponseFunctionWebSearchParam(TypedDict, total=False):
    id: Required[str]

    action: Required[InputUnionMember1ResponseFunctionWebSearchParamAction]

    status: Required[Literal["in_progress", "searching", "completed", "failed"]]

    type: Required[Literal["web_search_call"]]


class InputUnionMember1ResponseFunctionToolCallParam(TypedDict, total=False):
    arguments: Required[str]

    call_id: Required[str]

    name: Required[str]

    type: Required[Literal["function_call"]]

    id: str

    status: Literal["in_progress", "completed", "incomplete"]


class InputUnionMember1OpenAITypesResponsesResponseInputParamFunctionCallOutput(TypedDict, total=False):
    call_id: Required[str]

    output: Required[str]

    type: Required[Literal["function_call_output"]]

    id: str

    status: Literal["in_progress", "completed", "incomplete"]


class InputUnionMember1ResponseReasoningItemParamSummary(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["summary_text"]]


class InputUnionMember1ResponseReasoningItemParamContent(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["reasoning_text"]]


class InputUnionMember1ResponseReasoningItemParam(TypedDict, total=False):
    id: Required[str]

    summary: Required[Iterable[InputUnionMember1ResponseReasoningItemParamSummary]]

    type: Required[Literal["reasoning"]]

    content: Iterable[InputUnionMember1ResponseReasoningItemParamContent]

    encrypted_content: str

    status: Literal["in_progress", "completed", "incomplete"]


class InputUnionMember1OpenAITypesResponsesResponseInputParamImageGenerationCall(TypedDict, total=False):
    id: Required[str]

    result: Required[str]

    status: Required[Literal["in_progress", "completed", "generating", "failed"]]

    type: Required[Literal["image_generation_call"]]


class InputUnionMember1ResponseCodeInterpreterToolCallParamOutputOpenAITypesResponsesResponseCodeInterpreterToolCallParamOutputLogs(
    TypedDict, total=False
):
    logs: Required[str]

    type: Required[Literal["logs"]]


class InputUnionMember1ResponseCodeInterpreterToolCallParamOutputOpenAITypesResponsesResponseCodeInterpreterToolCallParamOutputImage(
    TypedDict, total=False
):
    type: Required[Literal["image"]]

    url: Required[str]


InputUnionMember1ResponseCodeInterpreterToolCallParamOutput: TypeAlias = Union[
    InputUnionMember1ResponseCodeInterpreterToolCallParamOutputOpenAITypesResponsesResponseCodeInterpreterToolCallParamOutputLogs,
    InputUnionMember1ResponseCodeInterpreterToolCallParamOutputOpenAITypesResponsesResponseCodeInterpreterToolCallParamOutputImage,
]


class InputUnionMember1ResponseCodeInterpreterToolCallParam(TypedDict, total=False):
    id: Required[str]

    code: Required[str]

    container_id: Required[str]

    outputs: Required[Iterable[InputUnionMember1ResponseCodeInterpreterToolCallParamOutput]]

    status: Required[Literal["in_progress", "completed", "incomplete", "interpreting", "failed"]]

    type: Required[Literal["code_interpreter_call"]]


class InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCallAction(TypedDict, total=False):
    command: Required[SequenceNotStr[str]]

    env: Required[Dict[str, str]]

    type: Required[Literal["exec"]]

    timeout_ms: int

    user: str

    working_directory: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCall(TypedDict, total=False):
    id: Required[str]

    action: Required[InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCallAction]

    call_id: Required[str]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["local_shell_call"]]


class InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCallOutput(TypedDict, total=False):
    id: Required[str]

    output: Required[str]

    type: Required[Literal["local_shell_call_output"]]

    status: Literal["in_progress", "completed", "incomplete"]


class InputUnionMember1OpenAITypesResponsesResponseInputParamMcpListToolsTool(TypedDict, total=False):
    input_schema: Required[object]

    name: Required[str]

    annotations: object

    description: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamMcpListTools(TypedDict, total=False):
    id: Required[str]

    server_label: Required[str]

    tools: Required[Iterable[InputUnionMember1OpenAITypesResponsesResponseInputParamMcpListToolsTool]]

    type: Required[Literal["mcp_list_tools"]]

    error: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamMcpApprovalRequest(TypedDict, total=False):
    id: Required[str]

    arguments: Required[str]

    name: Required[str]

    server_label: Required[str]

    type: Required[Literal["mcp_approval_request"]]


class InputUnionMember1OpenAITypesResponsesResponseInputParamMcpApprovalResponse(TypedDict, total=False):
    approval_request_id: Required[str]

    approve: Required[bool]

    type: Required[Literal["mcp_approval_response"]]

    id: str

    reason: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamMcpCall(TypedDict, total=False):
    id: Required[str]

    arguments: Required[str]

    name: Required[str]

    server_label: Required[str]

    type: Required[Literal["mcp_call"]]

    error: str

    output: str


class InputUnionMember1ResponseCustomToolCallOutputParam(TypedDict, total=False):
    call_id: Required[str]

    output: Required[str]

    type: Required[Literal["custom_tool_call_output"]]

    id: str


class InputUnionMember1ResponseCustomToolCallParam(TypedDict, total=False):
    call_id: Required[str]

    input: Required[str]

    name: Required[str]

    type: Required[Literal["custom_tool_call"]]

    id: str


class InputUnionMember1OpenAITypesResponsesResponseInputParamItemReference(TypedDict, total=False):
    id: Required[str]

    type: Literal["item_reference"]


InputUnionMember1: TypeAlias = Union[
    InputUnionMember1EasyInputMessageParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamMessage,
    InputUnionMember1ResponseOutputMessageParam,
    InputUnionMember1ResponseFileSearchToolCallParam,
    InputUnionMember1ResponseComputerToolCallParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamComputerCallOutput,
    InputUnionMember1ResponseFunctionWebSearchParam,
    InputUnionMember1ResponseFunctionToolCallParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamFunctionCallOutput,
    InputUnionMember1ResponseReasoningItemParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamImageGenerationCall,
    InputUnionMember1ResponseCodeInterpreterToolCallParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCall,
    InputUnionMember1OpenAITypesResponsesResponseInputParamLocalShellCallOutput,
    InputUnionMember1OpenAITypesResponsesResponseInputParamMcpListTools,
    InputUnionMember1OpenAITypesResponsesResponseInputParamMcpApprovalRequest,
    InputUnionMember1OpenAITypesResponsesResponseInputParamMcpApprovalResponse,
    InputUnionMember1OpenAITypesResponsesResponseInputParamMcpCall,
    InputUnionMember1ResponseCustomToolCallOutputParam,
    InputUnionMember1ResponseCustomToolCallParam,
    InputUnionMember1OpenAITypesResponsesResponseInputParamItemReference,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types import (
    DatasetItem,
    DatasetItemDeleteResponse,
    DatasetItemBatchCreateResponse,
)
from scale_gp_beta.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDatasetItems:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.retrieve(
            dataset_item_id="dataset_item_id",
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.retrieve(
            dataset_item_id="dataset_item_id",
            version=0,
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.dataset_items.with_raw_response.retrieve(
            dataset_item_id="dataset_item_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = response.parse()
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.dataset_items.with_streaming_response.retrieve(
            dataset_item_id="dataset_item_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = response.parse()
            assert_matches_type(DatasetItem, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_item_id` but received ''"):
            client.dataset_items.with_raw_response.retrieve(
                dataset_item_id="",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
            files={"foo": "string"},
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.dataset_items.with_raw_response.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = response.parse()
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.dataset_items.with_streaming_response.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = response.parse()
            assert_matches_type(DatasetItem, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_item_id` but received ''"):
            client.dataset_items.with_raw_response.update(
                dataset_item_id="",
                data={"foo": "bar"},
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.list()
        assert_matches_type(SyncCursorPage[DatasetItem], dataset_item, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.list(
            dataset_id="dataset_id",
            ending_before="ending_before",
            include_archived=True,
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
            version=0,
        )
        assert_matches_type(SyncCursorPage[DatasetItem], dataset_item, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.dataset_items.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = response.parse()
        assert_matches_type(SyncCursorPage[DatasetItem], dataset_item, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.dataset_items.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = response.parse()
            assert_matches_type(SyncCursorPage[DatasetItem], dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.delete(
            "dataset_item_id",
        )
        assert_matches_type(DatasetItemDeleteResponse, dataset_item, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.dataset_items.with_raw_response.delete(
            "dataset_item_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = response.parse()
        assert_matches_type(DatasetItemDeleteResponse, dataset_item, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.dataset_items.with_streaming_response.delete(
            "dataset_item_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = response.parse()
            assert_matches_type(DatasetItemDeleteResponse, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_item_id` but received ''"):
            client.dataset_items.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_batch_create(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
        )
        assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

    @parametrize
    def test_method_batch_create_with_all_params(self, client: SGPClient) -> None:
        dataset_item = client.dataset_items.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
            files=[{"foo": "string"}],
        )
        assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

    @parametrize
    def test_raw_response_batch_create(self, client: SGPClient) -> None:
        response = client.dataset_items.with_raw_response.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = response.parse()
        assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

    @parametrize
    def test_streaming_response_batch_create(self, client: SGPClient) -> None:
        with client.dataset_items.with_streaming_response.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = response.parse()
            assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncDatasetItems:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.retrieve(
            dataset_item_id="dataset_item_id",
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.retrieve(
            dataset_item_id="dataset_item_id",
            version=0,
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.dataset_items.with_raw_response.retrieve(
            dataset_item_id="dataset_item_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = await response.parse()
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.dataset_items.with_streaming_response.retrieve(
            dataset_item_id="dataset_item_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = await response.parse()
            assert_matches_type(DatasetItem, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_item_id` but received ''"):
            await async_client.dataset_items.with_raw_response.retrieve(
                dataset_item_id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
            files={"foo": "string"},
        )
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.dataset_items.with_raw_response.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = await response.parse()
        assert_matches_type(DatasetItem, dataset_item, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.dataset_items.with_streaming_response.update(
            dataset_item_id="dataset_item_id",
            data={"foo": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = await response.parse()
            assert_matches_type(DatasetItem, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_item_id` but received ''"):
            await async_client.dataset_items.with_raw_response.update(
                dataset_item_id="",
                data={"foo": "bar"},
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.list()
        assert_matches_type(AsyncCursorPage[DatasetItem], dataset_item, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.list(
            dataset_id="dataset_id",
            ending_before="ending_before",
            include_archived=True,
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
            version=0,
        )
        assert_matches_type(AsyncCursorPage[DatasetItem], dataset_item, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.dataset_items.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = await response.parse()
        assert_matches_type(AsyncCursorPage[DatasetItem], dataset_item, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.dataset_items.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = await response.parse()
            assert_matches_type(AsyncCursorPage[DatasetItem], dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.delete(
            "dataset_item_id",
        )
        assert_matches_type(DatasetItemDeleteResponse, dataset_item, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.dataset_items.with_raw_response.delete(
            "dataset_item_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = await response.parse()
        assert_matches_type(DatasetItemDeleteResponse, dataset_item, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.dataset_items.with_streaming_response.delete(
            "dataset_item_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = await response.parse()
            assert_matches_type(DatasetItemDeleteResponse, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dataset_item_id` but received ''"):
            await async_client.dataset_items.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_batch_create(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
        )
        assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

    @parametrize
    async def test_method_batch_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        dataset_item = await async_client.dataset_items.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
            files=[{"foo": "string"}],
        )
        assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

    @parametrize
    async def test_raw_response_batch_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.dataset_items.with_raw_response.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dataset_item = await response.parse()
        assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

    @parametrize
    async def test_streaming_response_batch_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.dataset_items.with_streaming_response.batch_create(
            data=[{"foo": "bar"}],
            dataset_id="dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dataset_item = await response.parse()
            assert_matches_type(DatasetItemBatchCreateResponse, dataset_item, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import TYPE_CHECKING, Dict, List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "Completion",
    "Choice",
    "ChoiceLogprobs",
    "Usage",
    "UsageCompletionTokensDetails",
    "UsagePromptTokensDetails",
]


class ChoiceLogprobs(BaseModel):
    text_offset: Optional[List[int]] = None

    token_logprobs: Optional[List[float]] = None

    tokens: Optional[List[str]] = None

    top_logprobs: Optional[List[Dict[str, float]]] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Choice(BaseModel):
    finish_reason: Literal["stop", "length", "content_filter"]

    index: int

    text: str

    logprobs: Optional[ChoiceLogprobs] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class UsageCompletionTokensDetails(BaseModel):
    accepted_prediction_tokens: Optional[int] = None

    audio_tokens: Optional[int] = None

    reasoning_tokens: Optional[int] = None

    rejected_prediction_tokens: Optional[int] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class UsagePromptTokensDetails(BaseModel):
    audio_tokens: Optional[int] = None

    cached_tokens: Optional[int] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Usage(BaseModel):
    completion_tokens: int

    prompt_tokens: int

    total_tokens: int

    completion_tokens_details: Optional[UsageCompletionTokensDetails] = None

    prompt_tokens_details: Optional[UsagePromptTokensDetails] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Completion(BaseModel):
    id: str

    choices: List[Choice]

    created: int

    model: str

    object: Optional[Literal["text_completion"]] = None

    system_fingerprint: Optional[str] = None

    usage: Optional[Usage] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, builtins.object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> builtins.object: ...
    else:
        __pydantic_extra__: Dict[str, builtins.object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .dataset_item import DatasetItem

__all__ = ["DatasetItemBatchCreateResponse"]


class DatasetItemBatchCreateResponse(BaseModel):
    items: List[DatasetItem]

    object: Optional[Literal["list"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EvaluationItemExportParams"]


class EvaluationItemExportParams(TypedDict, total=False):
    evaluation_id: Required[str]
    """The ID of the evaluation to export items from."""

    export_format: Literal["json", "jsonl", "csv"]
    """The format of the exported evaluation items.

    `json` returns a single JSON array, while `jsonl` returns one JSON object per
    line.
    """

    export_method: Literal["signed_url", "direct"]
    """The method for exporting evaluation items.

    `signed_url` returns a pre-signed URL, while `direct` returns the raw content.
    """

    include_archived: bool
    """If true, include archived evaluation items in the export."""



"""
Simple fibonacci tracing example. To run, first fill-in account_id and api key.

Using low-level Trace and Span objects.
"""

import time

import scale_gp_beta.lib.tracing as tracing
from scale_gp_beta import SGPClient


def fibonacci(curr: int) -> int:
    with tracing.create_span("fibonacci", input={"curr": curr}) as span:
        time.sleep(0.1)
        if curr < 2:
            span.output = {"res": curr}
            return curr

        res = fibonacci(curr - 1) + fibonacci(curr - 2)
        span.output = {"res": res}
    return res


def main() -> None:
    # create traces and spans with create_trace and create_span
    # can act directly or use context managers
    with tracing.create_trace("my_trace"):
        span = tracing.create_span("main", input={}, metadata={"env": "local"})
        span.start()

        fib = fibonacci(5)

        span.output = {"result": fib}
        span.end()


if __name__ == "__main__":
    api_key = "XXX"
    account_id = "XXX"

    # Initialise with a working client, can omit if two appropriate ENV VARs are set
    tracing.init(SGPClient(api_key=api_key, account_id=account_id), disabled=False)
    main()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["InferenceResponse"]


class InferenceResponse(BaseModel):
    response: object

    object: Optional[Literal["generic_inference"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["CompletionCreateParamsBase", "CompletionCreateParamsNonStreaming", "CompletionCreateParamsStreaming"]


class CompletionCreateParamsBase(TypedDict, total=False):
    messages: Required[Iterable[Dict[str, object]]]
    """openai standard message format"""

    model: Required[str]
    """model specified as `model_vendor/model`, for example `openai/gpt-4o`"""

    audio: Dict[str, object]
    """Parameters for audio output.

    Required when audio output is requested with modalities: ['audio'].
    """

    frequency_penalty: float
    """Number between -2.0 and 2.0.

    Positive values penalize new tokens based on their existing frequency in the
    text so far.
    """

    function_call: Dict[str, object]
    """Deprecated in favor of tool_choice.

    Controls which function is called by the model.
    """

    functions: Iterable[Dict[str, object]]
    """Deprecated in favor of tools.

    A list of functions the model may generate JSON inputs for.
    """

    logit_bias: Dict[str, int]
    """Modify the likelihood of specified tokens appearing in the completion.

    Maps tokens to bias values from -100 to 100.
    """

    logprobs: bool
    """Whether to return log probabilities of the output tokens or not."""

    max_completion_tokens: int
    """
    An upper bound for the number of tokens that can be generated, including visible
    output tokens and reasoning tokens.
    """

    max_tokens: int
    """Deprecated in favor of max_completion_tokens.

    The maximum number of tokens to generate.
    """

    metadata: Dict[str, str]
    """
    Developer-defined tags and values used for filtering completions in the
    dashboard.
    """

    modalities: SequenceNotStr[str]
    """Output types that you would like the model to generate for this request."""

    n: int
    """How many chat completion choices to generate for each input message."""

    parallel_tool_calls: bool
    """Whether to enable parallel function calling during tool use."""

    prediction: Dict[str, object]
    """
    Static predicted output content, such as the content of a text file being
    regenerated.
    """

    presence_penalty: float
    """Number between -2.0 and 2.0.

    Positive values penalize tokens based on whether they appear in the text so far.
    """

    reasoning_effort: str
    """For o1 models only. Constrains effort on reasoning. Values: low, medium, high."""

    response_format: Dict[str, object]
    """An object specifying the format that the model must output."""

    seed: int
    """
    If specified, system will attempt to sample deterministically for repeated
    requests with same seed.
    """

    stop: Union[str, SequenceNotStr[str]]
    """Up to 4 sequences where the API will stop generating further tokens."""

    store: bool
    """Whether to store the output for use in model distillation or evals products."""

    stream_options: Dict[str, object]
    """Options for streaming response. Only set this when stream is true."""

    temperature: float
    """What sampling temperature to use.

    Higher values make output more random, lower more focused.
    """

    tool_choice: Union[str, Dict[str, object]]
    """Controls which tool is called by the model.

    Values: none, auto, required, or specific tool.
    """

    tools: Iterable[Dict[str, object]]
    """A list of tools the model may call.

    Currently, only functions are supported. Max 128 functions.
    """

    top_k: int
    """Only sample from the top K options for each subsequent token"""

    top_logprobs: int
    """
    Number of most likely tokens to return at each position, with associated log
    probability.
    """

    top_p: float
    """Alternative to temperature.

    Only tokens comprising top_p probability mass are considered.
    """


class CompletionCreateParamsNonStreaming(CompletionCreateParamsBase, total=False):
    stream: Literal[False]
    """If true, partial message deltas will be sent as server-sent events."""


class CompletionCreateParamsStreaming(CompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """If true, partial message deltas will be sent as server-sent events."""


CompletionCreateParams = Union[CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "EvaluationCreateParams",
    "EvaluationStandaloneCreateRequest",
    "EvaluationFromDatasetCreateRequest",
    "EvaluationFromDatasetCreateRequestData",
    "EvaluationWithDatasetCreateRequest",
    "EvaluationWithDatasetCreateRequestDataset",
]


class EvaluationStandaloneCreateRequest(TypedDict, total=False):
    data: Required[Iterable[Dict[str, object]]]
    """Items to be evaluated"""

    name: Required[str]

    description: str

    files: Iterable[Dict[str, str]]
    """Files to be associated to the evaluation"""

    metadata: Dict[str, object]
    """Optional metadata key-value pairs for the evaluation"""

    tags: SequenceNotStr[str]
    """The tags associated with the entity"""

    tasks: Iterable["EvaluationTaskParam"]
    """Tasks allow you to augment and evaluate your data"""


class EvaluationFromDatasetCreateRequest(TypedDict, total=False):
    dataset_id: Required[str]
    """The ID of the dataset containing the items referenced by the `data` field"""

    name: Required[str]

    data: Iterable[EvaluationFromDatasetCreateRequestData]
    """Items to be evaluated, including references to the input dataset"""

    description: str

    metadata: Dict[str, object]
    """Optional metadata key-value pairs for the evaluation"""

    tags: SequenceNotStr[str]
    """The tags associated with the entity"""

    tasks: Iterable["EvaluationTaskParam"]
    """Tasks allow you to augment and evaluate your data"""


class EvaluationFromDatasetCreateRequestDataTyped(TypedDict, total=False):
    dataset_item_id: Required[str]


EvaluationFromDatasetCreateRequestData: TypeAlias = Union[
    EvaluationFromDatasetCreateRequestDataTyped, Dict[str, object]
]


class EvaluationWithDatasetCreateRequest(TypedDict, total=False):
    data: Required[Iterable[Dict[str, object]]]
    """Items to be evaluated"""

    dataset: Required[EvaluationWithDatasetCreateRequestDataset]
    """Create a reusable dataset from items in the `data` field"""

    name: Required[str]

    description: str

    files: Iterable[Dict[str, str]]
    """Files to be associated to the evaluation"""

    metadata: Dict[str, object]
    """Optional metadata key-value pairs for the evaluation"""

    tags: SequenceNotStr[str]
    """The tags associated with the entity"""

    tasks: Iterable["EvaluationTaskParam"]
    """Tasks allow you to augment and evaluate your data"""


class EvaluationWithDatasetCreateRequestDataset(TypedDict, total=False):
    """Create a reusable dataset from items in the `data` field"""

    name: Required[str]

    description: str

    keys: SequenceNotStr[str]
    """Keys from items in the `data` field that should be included in the dataset.

    If not provided, all keys will be included.
    """

    tags: SequenceNotStr[str]
    """The tags associated with the entity"""


EvaluationCreateParams: TypeAlias = Union[
    EvaluationStandaloneCreateRequest, EvaluationFromDatasetCreateRequest, EvaluationWithDatasetCreateRequest
]

from .evaluation_task_param import EvaluationTaskParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["VectorStoreCreateParams"]


class VectorStoreCreateParams(TypedDict, total=False):
    embedding_model: Required[
        Literal[
            "sentence-transformers/all-MiniLM-L12-v2",
            "sentence-transformers/all-mpnet-base-v2",
            "sentence-transformers/multi-qa-distilbert-cos-v1",
            "sentence-transformers/paraphrase-multilingual-mpnet-base-v2",
            "openai/text-embedding-ada-002",
            "openai/text-embedding-3-small",
            "openai/text-embedding-3-large",
            "embed-english-v3.0",
            "embed-english-light-v3.0",
            "embed-multilingual-v3.0",
        ]
    ]
    """The embedding model to use"""

    name: Required[str]
    """A unique name for the vector store within the account"""

    indexed_metadata_fields: Dict[str, Literal["string", "number", "boolean", "object", "list"]]
    """Dictionary mapping metadata field names to their types for efficient filtering.

    Only STRING, NUMBER, and BOOLEAN types can be indexed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .shared.identity import Identity

__all__ = [
    "Question",
    "CategoricalQuestion",
    "CategoricalQuestionConfiguration",
    "RatingQuestion",
    "RatingQuestionConfiguration",
    "NumberQuestion",
    "NumberQuestionConfiguration",
    "FreeTextQuestion",
    "FreeTextQuestionConfiguration",
    "FormQuestion",
    "FormQuestionConfiguration",
    "TimestampQuestion",
    "TimestampQuestionConfiguration",
]


class CategoricalQuestionConfiguration(BaseModel):
    choices: List[str]
    """Categorical answer choices (must contain at least one entry)"""

    dropdown: Optional[bool] = None
    """Whether the question is displayed as a dropdown in the UI."""

    multi: Optional[bool] = None
    """Whether the question allows multiple answers."""


class CategoricalQuestion(BaseModel):
    id: str
    """Unique identifier of the entity"""

    configuration: CategoricalQuestionConfiguration

    created_at: datetime
    """ISO-timestamp when the entity was created"""

    created_by: Identity
    """The identity that created the entity."""

    name: str

    prompt: str
    """user-facing question prompt"""

    conditions: Optional[List[Dict[str, object]]] = None
    """Conditions for the question to be shown"""

    object: Optional[Literal["question"]] = None

    question_type: Optional[Literal["categorical"]] = None


class RatingQuestionConfiguration(BaseModel):
    max_label: str
    """Label shown for the maximum rating"""

    min_label: str
    """Label shown for the minimum rating"""

    steps: int
    """Number of discrete points on the scale (e.g., 5 for a 1–5 scale)"""


class RatingQuestion(BaseModel):
    id: str
    """Unique identifier of the entity"""

    configuration: RatingQuestionConfiguration

    created_at: datetime
    """ISO-timestamp when the entity was created"""

    created_by: Identity
    """The identity that created the entity."""

    name: str

    prompt: str
    """user-facing question prompt"""

    conditions: Optional[List[Dict[str, object]]] = None
    """Conditions for the question to be shown"""

    object: Optional[Literal["question"]] = None

    question_type: Optional[Literal["rating"]] = None


class NumberQuestionConfiguration(BaseModel):
    max: Optional[float] = None
    """Maximum value for the number"""

    min: Optional[float] = None
    """Minimum value for the number"""


class NumberQuestion(BaseModel):
    id: str
    """Unique identifier of the entity"""

    created_at: datetime
    """ISO-timestamp when the entity was created"""

    created_by: Identity
    """The identity that created the entity."""

    name: str

    prompt: str
    """user-facing question prompt"""

    conditions: Optional[List[Dict[str, object]]] = None
    """Conditions for the question to be shown"""

    configuration: Optional[NumberQuestionConfiguration] = None

    object: Optional[Literal["question"]] = None

    question_type: Optional[Literal["number"]] = None


class FreeTextQuestionConfiguration(BaseModel):
    max_length: Optional[int] = None
    """Maximum characters allowed"""

    min_length: Optional[int] = None
    """Minimum characters required"""


class FreeTextQuestion(BaseModel):
    id: str
    """Unique identifier of the entity"""

    created_at: datetime
    """ISO-timestamp when the entity was created"""

    created_by: Identity
    """The identity that created the entity."""

    name: str

    prompt: str
    """user-facing question prompt"""

    conditions: Optional[List[Dict[str, object]]] = None
    """Conditions for the question to be shown"""

    configuration: Optional[FreeTextQuestionConfiguration] = None

    object: Optional[Literal["question"]] = None

    question_type: Optional[Literal["free_text"]] = None


class FormQuestionConfiguration(BaseModel):
    form_schema: Dict[str, object]
    """The JSON schema of the desired form object"""


class FormQuestion(BaseModel):
    id: str
    """Unique identifier of the entity"""

    configuration: FormQuestionConfiguration

    created_at: datetime
    """ISO-timestamp when the entity was created"""

    created_by: Identity
    """The identity that created the entity."""

    name: str

    prompt: str
    """user-facing question prompt"""

    conditions: Optional[List[Dict[str, object]]] = None
    """Conditions for the question to be shown"""

    object: Optional[Literal["question"]] = None

    question_type: Optional[Literal["form"]] = None


class TimestampQuestionConfiguration(BaseModel):
    multi: Optional[bool] = None
    """Whether to allow multiple timestamps"""


class TimestampQuestion(BaseModel):
    id: str
    """Unique identifier of the entity"""

    created_at: datetime
    """ISO-timestamp when the entity was created"""

    created_by: Identity
    """The identity that created the entity."""

    name: str

    prompt: str
    """user-facing question prompt"""

    conditions: Optional[List[Dict[str, object]]] = None
    """Conditions for the question to be shown"""

    configuration: Optional[TimestampQuestionConfiguration] = None

    object: Optional[Literal["question"]] = None

    question_type: Optional[Literal["timestamp"]] = None


Question: TypeAlias = Annotated[
    Union[CategoricalQuestion, RatingQuestion, NumberQuestion, FreeTextQuestion, FormQuestion, TimestampQuestion],
    PropertyInfo(discriminator="question_type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["InferenceCreateParams", "InferenceConfiguration"]


class InferenceCreateParams(TypedDict, total=False):
    model: Required[str]
    """model specified as `vendor/name` (ex. openai/gpt-5)"""

    args: Dict[str, object]
    """Arguments passed into model"""

    inference_configuration: InferenceConfiguration
    """Vendor specific configuration"""


class InferenceConfiguration(TypedDict, total=False):
    """Vendor specific configuration"""

    num_retries: int

    timeout_seconds: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types.evaluation_dashboards import (
    EvaluationDashboardWidgetWithResult,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWidgets:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        widget = client.evaluation_dashboards.widgets.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        widget = client.evaluation_dashboards.widgets.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
            config={"foo": "bar"},
            query={
                "select": [
                    {
                        "expression": {
                            "column": "column",
                            "source": "source",
                            "type": "COLUMN",
                        },
                        "alias": "alias",
                    }
                ],
                "filter": {
                    "conditions": [
                        {
                            "column": "column",
                            "operator": "=",
                            "value": {},
                            "source": "source",
                        }
                    ],
                    "logical_operators": ["AND"],
                },
                "group_by": ["string"],
                "limit": 1,
                "order_by": [
                    {
                        "column": "column",
                        "direction": "ASC",
                        "source": "source",
                    }
                ],
            },
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.widgets.with_raw_response.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        widget = response.parse()
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.widgets.with_streaming_response.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            widget = response.parse()
            assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            client.evaluation_dashboards.widgets.with_raw_response.create(
                dashboard_id="",
                title="x",
                type="bar",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        widget = client.evaluation_dashboards.widgets.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        widget = client.evaluation_dashboards.widgets.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
            config={"foo": "bar"},
            query={
                "select": [
                    {
                        "expression": {
                            "column": "column",
                            "source": "source",
                            "type": "COLUMN",
                        },
                        "alias": "alias",
                    }
                ],
                "filter": {
                    "conditions": [
                        {
                            "column": "column",
                            "operator": "=",
                            "value": {},
                            "source": "source",
                        }
                    ],
                    "logical_operators": ["AND"],
                },
                "group_by": ["string"],
                "limit": 1,
                "order_by": [
                    {
                        "column": "column",
                        "direction": "ASC",
                        "source": "source",
                    }
                ],
            },
            title="x",
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.widgets.with_raw_response.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        widget = response.parse()
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.widgets.with_streaming_response.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            widget = response.parse()
            assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            client.evaluation_dashboards.widgets.with_raw_response.update(
                widget_id="widget_id",
                dashboard_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `widget_id` but received ''"):
            client.evaluation_dashboards.widgets.with_raw_response.update(
                widget_id="",
                dashboard_id="dashboard_id",
            )

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        widget = client.evaluation_dashboards.widgets.delete(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )
        assert widget is None

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.evaluation_dashboards.widgets.with_raw_response.delete(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        widget = response.parse()
        assert widget is None

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.evaluation_dashboards.widgets.with_streaming_response.delete(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            widget = response.parse()
            assert widget is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            client.evaluation_dashboards.widgets.with_raw_response.delete(
                widget_id="widget_id",
                dashboard_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `widget_id` but received ''"):
            client.evaluation_dashboards.widgets.with_raw_response.delete(
                widget_id="",
                dashboard_id="dashboard_id",
            )


class TestAsyncWidgets:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        widget = await async_client.evaluation_dashboards.widgets.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        widget = await async_client.evaluation_dashboards.widgets.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
            config={"foo": "bar"},
            query={
                "select": [
                    {
                        "expression": {
                            "column": "column",
                            "source": "source",
                            "type": "COLUMN",
                        },
                        "alias": "alias",
                    }
                ],
                "filter": {
                    "conditions": [
                        {
                            "column": "column",
                            "operator": "=",
                            "value": {},
                            "source": "source",
                        }
                    ],
                    "logical_operators": ["AND"],
                },
                "group_by": ["string"],
                "limit": 1,
                "order_by": [
                    {
                        "column": "column",
                        "direction": "ASC",
                        "source": "source",
                    }
                ],
            },
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.widgets.with_raw_response.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        widget = await response.parse()
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.widgets.with_streaming_response.create(
            dashboard_id="dashboard_id",
            title="x",
            type="bar",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            widget = await response.parse()
            assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            await async_client.evaluation_dashboards.widgets.with_raw_response.create(
                dashboard_id="",
                title="x",
                type="bar",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        widget = await async_client.evaluation_dashboards.widgets.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        widget = await async_client.evaluation_dashboards.widgets.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
            config={"foo": "bar"},
            query={
                "select": [
                    {
                        "expression": {
                            "column": "column",
                            "source": "source",
                            "type": "COLUMN",
                        },
                        "alias": "alias",
                    }
                ],
                "filter": {
                    "conditions": [
                        {
                            "column": "column",
                            "operator": "=",
                            "value": {},
                            "source": "source",
                        }
                    ],
                    "logical_operators": ["AND"],
                },
                "group_by": ["string"],
                "limit": 1,
                "order_by": [
                    {
                        "column": "column",
                        "direction": "ASC",
                        "source": "source",
                    }
                ],
            },
            title="x",
        )
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.widgets.with_raw_response.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        widget = await response.parse()
        assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.widgets.with_streaming_response.update(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            widget = await response.parse()
            assert_matches_type(EvaluationDashboardWidgetWithResult, widget, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            await async_client.evaluation_dashboards.widgets.with_raw_response.update(
                widget_id="widget_id",
                dashboard_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `widget_id` but received ''"):
            await async_client.evaluation_dashboards.widgets.with_raw_response.update(
                widget_id="",
                dashboard_id="dashboard_id",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        widget = await async_client.evaluation_dashboards.widgets.delete(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )
        assert widget is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_dashboards.widgets.with_raw_response.delete(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        widget = await response.parse()
        assert widget is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_dashboards.widgets.with_streaming_response.delete(
            widget_id="widget_id",
            dashboard_id="dashboard_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            widget = await response.parse()
            assert widget is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `dashboard_id` but received ''"):
            await async_client.evaluation_dashboards.widgets.with_raw_response.delete(
                widget_id="widget_id",
                dashboard_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `widget_id` but received ''"):
            await async_client.evaluation_dashboards.widgets.with_raw_response.delete(
                widget_id="",
                dashboard_id="dashboard_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "InferenceModel",
    "VendorConfiguration",
    "VendorConfigurationLaunchVendorConfiguration",
    "VendorConfigurationLaunchVendorConfigurationModelImage",
    "VendorConfigurationLaunchVendorConfigurationModelInfra",
    "VendorConfigurationLlmEngineVendorConfiguration",
]


class VendorConfigurationLaunchVendorConfigurationModelImage(BaseModel):
    command: List[str]

    registry: str

    repository: str

    tag: str

    env_vars: Optional[Dict[str, object]] = None

    healthcheck_route: Optional[str] = None

    predict_route: Optional[str] = None

    readiness_delay: Optional[int] = None

    request_schema: Optional[Dict[str, object]] = None

    response_schema: Optional[Dict[str, object]] = None

    streaming_command: Optional[List[str]] = None

    streaming_predict_route: Optional[str] = None


class VendorConfigurationLaunchVendorConfigurationModelInfra(BaseModel):
    cpus: Union[str, int, None] = None

    endpoint_type: Optional[Literal["async", "sync", "streaming"]] = None

    gpu_type: Optional[
        Literal[
            "nvidia-tesla-t4",
            "nvidia-ampere-a10",
            "nvidia-ampere-a100",
            "nvidia-ampere-a100e",
            "nvidia-hopper-h100",
            "nvidia-hopper-h100-1g20gb",
            "nvidia-hopper-h100-3g40gb",
        ]
    ] = None

    gpus: Optional[int] = None

    high_priority: Optional[bool] = None

    labels: Optional[Dict[str, str]] = None

    max_workers: Optional[int] = None

    memory: Optional[str] = None

    min_workers: Optional[int] = None

    per_worker: Optional[int] = None

    public_inference: Optional[bool] = None

    storage: Optional[str] = None


class VendorConfigurationLaunchVendorConfiguration(BaseModel):
    image: VendorConfigurationLaunchVendorConfigurationModelImage = FieldInfo(alias="model_image")

    infra: VendorConfigurationLaunchVendorConfigurationModelInfra = FieldInfo(alias="model_infra")


class VendorConfigurationLlmEngineVendorConfiguration(BaseModel):
    model: str

    checkpoint_path: Optional[str] = None

    cpus: Optional[int] = None

    default_callback_url: Optional[str] = None

    endpoint_type: Optional[str] = None

    gpu_type: Optional[str] = None

    gpus: Optional[int] = None

    high_priority: Optional[bool] = None

    inference_framework: Optional[str] = None

    inference_framework_image_tag: Optional[str] = None

    labels: Optional[Dict[str, str]] = None

    max_workers: Optional[int] = None

    memory: Optional[str] = None

    min_workers: Optional[int] = None

    nodes_per_worker: Optional[int] = None

    num_shards: Optional[int] = None

    per_worker: Optional[int] = None

    post_inference_hooks: Optional[List[str]] = None

    public_inference: Optional[bool] = None

    quantize: Optional[str] = None

    source: Optional[str] = None

    storage: Optional[str] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


VendorConfiguration: TypeAlias = Union[
    VendorConfigurationLaunchVendorConfiguration, VendorConfigurationLlmEngineVendorConfiguration
]


class InferenceModel(BaseModel):
    id: str

    created_at: datetime

    created_by_identity_type: Literal["user", "service_account"]

    created_by_user_id: str

    type: Literal["generic", "completion", "chat_completion"] = FieldInfo(alias="model_type")

    vendor: Literal[
        "openai",
        "cohere",
        "vertex_ai",
        "anthropic",
        "azure",
        "gemini",
        "launch",
        "llmengine",
        "model_zoo",
        "bedrock",
        "xai",
        "fireworks_ai",
    ] = FieldInfo(alias="model_vendor")

    name: str

    status: Literal["failed", "ready", "deploying"]

    availability: Optional[Literal["unknown", "available", "unavailable"]] = FieldInfo(
        alias="model_availability", default=None
    )

    metadata: Optional[Dict[str, object]] = FieldInfo(alias="model_metadata", default=None)

    object: Optional[Literal["model"]] = None

    vendor_configuration: Optional[VendorConfiguration] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["SpanStatus"]

SpanStatus: TypeAlias = Literal["SUCCESS", "ERROR", "CANCELED"]



import uuid
from datetime import datetime, timezone, timedelta

import scale_gp_beta.lib.tracing as tracing
from scale_gp_beta import SGPClient


def main() -> None:
    parent_span_id = str(uuid.uuid4())
    trace_id = str(uuid.uuid4())
    child_span_id = str(uuid.uuid4())

    now = datetime.now(timezone.utc)

    start_time = (now - timedelta(minutes=10)).isoformat()
    end_time = now.isoformat()
    parent_span = tracing.create_span(
        "my_parent_span_name",
        input={"test": "input"},
        output={"test": "output"},
        metadata={"test": "metadata"},
        span_id=parent_span_id,
        trace_id=trace_id,
    )
    parent_span.start_time = start_time
    parent_span.end_time = end_time
    parent_span.flush()

    start_time = (now - timedelta(minutes=6)).isoformat()
    end_time = (now - timedelta(minutes=2)).isoformat()
    child_span = tracing.create_span(
        "my_child_span_name",
        input={"test": "another input"},
        output={"test": "another output"},
        metadata={"test": "another metadata"},
        span_id=child_span_id,
        trace_id=trace_id,
        parent_id=parent_span_id,
    )
    child_span.start_time = start_time
    child_span.end_time = end_time
    child_span.flush()


if __name__ == "__main__":
    api_key = "xxx"
    account_id = "xxx"
    tracing.init(SGPClient(api_key=api_key, account_id=account_id), disabled=False)
    main()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["VectorStoreDeleteParams"]


class VectorStoreDeleteParams(TypedDict, total=False):
    filter: Dict[str, object]
    """Metadata filter expression for deletion"""

    ids: SequenceNotStr[str]
    """Array of document IDs to delete"""



"""
Simple mock database query.

Explicit control example - no explicit traces just manually passing a trace_id.
"""
import time
import uuid
import random
from typing import Any, Dict

import scale_gp_beta.lib.tracing as tracing
from scale_gp_beta import SGPClient


class MockDatabase:
    """A mock database class to simulate queries."""
    def __init__(self) -> None:
        self._data = {
            "SELECT * FROM users WHERE id = 1;": {"id": 1, "name": "Alice"},
            "SELECT * FROM users WHERE id = 2;": {"id": 2, "name": "Bob"},
        }

    def execute_query(self, query: str, trace_id: str) -> Dict[str, Any]:
        with tracing.create_span("db_query", input={"query": query}, trace_id=trace_id) as span:
            # simulate delay
            time.sleep(random.uniform(0.1, 0.3))

            result = self._data.get(query, {})
            span.output = {"result": result}
            return result

def get_user_from_db(db: MockDatabase, user_id: int, trace_id: str) -> Dict[str, Any]:
    with tracing.create_span("get_user_from_db", input={"user_id": user_id}, trace_id=trace_id):
        query = f"SELECT * FROM users WHERE id = {user_id};"

        return db.execute_query(query, trace_id)

def main() -> None:
    db = MockDatabase()
    trace_id = str(uuid.uuid4())
    with tracing.create_span("main", metadata={"env": "local"}, trace_id=trace_id):
        user = get_user_from_db(db, 1, trace_id)
        print(f"Retrieved user: {user.get('name')}")

if __name__ == "__main__":
    api_key = "xxx"
    account_id = "xxx"
    tracing.init(SGPClient(api_key=api_key, account_id=account_id), disabled=False)
    main()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, Union, Iterable, cast
from typing_extensions import Literal

import httpx

from ..types import response_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.response_create_response import ResponseCreateResponse

__all__ = ["ResponsesResource", "AsyncResponsesResource"]


class ResponsesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ResponsesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return ResponsesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResponsesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return ResponsesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        input: Union[str, Iterable[response_create_params.InputUnionMember1]],
        model: str,
        include: SequenceNotStr[str] | Omit = omit,
        instructions: str | Omit = omit,
        max_output_tokens: int | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        previous_response_id: str | Omit = omit,
        reasoning: Dict[str, object] | Omit = omit,
        store: bool | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        text: Dict[str, object] | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_p: float | Omit = omit,
        truncation: Literal["auto", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseCreateResponse:
        """
        Responses

        Args:
          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          include: Which fields to include in the response

          instructions: Instructions for the response generation

          max_output_tokens: Maximum number of output tokens

          metadata: Metadata for the response

          parallel_tool_calls: Whether to enable parallel tool calls

          previous_response_id: ID of the previous response for chaining

          reasoning: Reasoning configuration for the response

          store: Whether to store the response

          stream: Whether to stream the response

          temperature: Sampling temperature for randomness control

          text: Text configuration parameters

          tool_choice: Tool choice configuration

          tools: Tools available for the response

          top_p: Top-p sampling parameter

          truncation: Truncation configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ResponseCreateResponse,
            self._post(
                "/v5/responses",
                body=maybe_transform(
                    {
                        "input": input,
                        "model": model,
                        "include": include,
                        "instructions": instructions,
                        "max_output_tokens": max_output_tokens,
                        "metadata": metadata,
                        "parallel_tool_calls": parallel_tool_calls,
                        "previous_response_id": previous_response_id,
                        "reasoning": reasoning,
                        "store": store,
                        "stream": stream,
                        "temperature": temperature,
                        "text": text,
                        "tool_choice": tool_choice,
                        "tools": tools,
                        "top_p": top_p,
                        "truncation": truncation,
                    },
                    response_create_params.ResponseCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ResponseCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncResponsesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncResponsesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncResponsesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResponsesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncResponsesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        input: Union[str, Iterable[response_create_params.InputUnionMember1]],
        model: str,
        include: SequenceNotStr[str] | Omit = omit,
        instructions: str | Omit = omit,
        max_output_tokens: int | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        parallel_tool_calls: bool | Omit = omit,
        previous_response_id: str | Omit = omit,
        reasoning: Dict[str, object] | Omit = omit,
        store: bool | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        text: Dict[str, object] | Omit = omit,
        tool_choice: Union[str, Dict[str, object]] | Omit = omit,
        tools: Iterable[Dict[str, object]] | Omit = omit,
        top_p: float | Omit = omit,
        truncation: Literal["auto", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseCreateResponse:
        """
        Responses

        Args:
          model: model specified as `model_vendor/model`, for example `openai/gpt-4o`

          include: Which fields to include in the response

          instructions: Instructions for the response generation

          max_output_tokens: Maximum number of output tokens

          metadata: Metadata for the response

          parallel_tool_calls: Whether to enable parallel tool calls

          previous_response_id: ID of the previous response for chaining

          reasoning: Reasoning configuration for the response

          store: Whether to store the response

          stream: Whether to stream the response

          temperature: Sampling temperature for randomness control

          text: Text configuration parameters

          tool_choice: Tool choice configuration

          tools: Tools available for the response

          top_p: Top-p sampling parameter

          truncation: Truncation configuration

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ResponseCreateResponse,
            await self._post(
                "/v5/responses",
                body=await async_maybe_transform(
                    {
                        "input": input,
                        "model": model,
                        "include": include,
                        "instructions": instructions,
                        "max_output_tokens": max_output_tokens,
                        "metadata": metadata,
                        "parallel_tool_calls": parallel_tool_calls,
                        "previous_response_id": previous_response_id,
                        "reasoning": reasoning,
                        "store": store,
                        "stream": stream,
                        "temperature": temperature,
                        "text": text,
                        "tool_choice": tool_choice,
                        "tools": tools,
                        "top_p": top_p,
                        "truncation": truncation,
                    },
                    response_create_params.ResponseCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ResponseCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class ResponsesResourceWithRawResponse:
    def __init__(self, responses: ResponsesResource) -> None:
        self._responses = responses

        self.create = to_raw_response_wrapper(
            responses.create,
        )


class AsyncResponsesResourceWithRawResponse:
    def __init__(self, responses: AsyncResponsesResource) -> None:
        self._responses = responses

        self.create = async_to_raw_response_wrapper(
            responses.create,
        )


class ResponsesResourceWithStreamingResponse:
    def __init__(self, responses: ResponsesResource) -> None:
        self._responses = responses

        self.create = to_streamed_response_wrapper(
            responses.create,
        )


class AsyncResponsesResourceWithStreamingResponse:
    def __init__(self, responses: AsyncResponsesResource) -> None:
        self._responses = responses

        self.create = async_to_streamed_response_wrapper(
            responses.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast
from typing_extensions import Literal

import httpx

from ..types import build_list_params, build_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._streaming import Stream, AsyncStream
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.stream_chunk import StreamChunk
from ..types.build_list_response import BuildListResponse
from ..types.build_cancel_response import BuildCancelResponse
from ..types.build_create_response import BuildCreateResponse
from ..types.build_retrieve_response import BuildRetrieveResponse

__all__ = ["BuildResource", "AsyncBuildResource"]


class BuildResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BuildResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return BuildResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BuildResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return BuildResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        context_archive: FileTypes,
        image_name: str,
        build_args: str | Omit = omit,
        image_tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BuildCreateResponse:
        """
        Submit a container image build.

        Upload a tar.gz archive containing the build context (Dockerfile and any files
        needed for the build) along with image name, tag, and optional build arguments.

        Maximum file size: 500MB

        Args:
          context_archive: tar.gz archive containing the build context (Dockerfile and any files needed for
              the build)

          image_name: Name for the built image

          build_args: JSON string of build arguments

          image_tag: Tag for the built image

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "context_archive": context_archive,
                "image_name": image_name,
                "build_args": build_args,
                "image_tag": image_tag,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["context_archive"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v5/builds",
            body=maybe_transform(body, build_create_params.BuildCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BuildCreateResponse,
        )

    def retrieve(
        self,
        build_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BuildRetrieveResponse:
        """
        Get a build by ID, including current status from the cloud provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not build_id:
            raise ValueError(f"Expected a non-empty value for `build_id` but received {build_id!r}")
        return self._get(
            f"/v5/builds/{build_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BuildRetrieveResponse,
        )

    def list(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[BuildListResponse]:
        """
        List Builds

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/builds",
            page=SyncCursorPage[BuildListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    build_list_params.BuildListParams,
                ),
            ),
            model=BuildListResponse,
        )

    def cancel(
        self,
        build_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BuildCancelResponse:
        """
        Cancel a pending or running build.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not build_id:
            raise ValueError(f"Expected a non-empty value for `build_id` but received {build_id!r}")
        return self._post(
            f"/v5/builds/{build_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BuildCancelResponse,
        )

    def logs(
        self,
        build_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[StreamChunk]:
        """
        Stream build logs via Server-Sent Events (SSE).

        Returns a streaming response with content-type text/event-stream. Each log line
        is sent as an SSE data event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not build_id:
            raise ValueError(f"Expected a non-empty value for `build_id` but received {build_id!r}")
        return self._get(
            f"/v5/builds/{build_id}/logs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StreamChunk,
            stream=True,
            stream_cls=Stream[StreamChunk],
        )


class AsyncBuildResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBuildResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncBuildResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBuildResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncBuildResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        context_archive: FileTypes,
        image_name: str,
        build_args: str | Omit = omit,
        image_tag: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BuildCreateResponse:
        """
        Submit a container image build.

        Upload a tar.gz archive containing the build context (Dockerfile and any files
        needed for the build) along with image name, tag, and optional build arguments.

        Maximum file size: 500MB

        Args:
          context_archive: tar.gz archive containing the build context (Dockerfile and any files needed for
              the build)

          image_name: Name for the built image

          build_args: JSON string of build arguments

          image_tag: Tag for the built image

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "context_archive": context_archive,
                "image_name": image_name,
                "build_args": build_args,
                "image_tag": image_tag,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["context_archive"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v5/builds",
            body=await async_maybe_transform(body, build_create_params.BuildCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BuildCreateResponse,
        )

    async def retrieve(
        self,
        build_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BuildRetrieveResponse:
        """
        Get a build by ID, including current status from the cloud provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not build_id:
            raise ValueError(f"Expected a non-empty value for `build_id` but received {build_id!r}")
        return await self._get(
            f"/v5/builds/{build_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BuildRetrieveResponse,
        )

    def list(
        self,
        *,
        ending_before: str | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BuildListResponse, AsyncCursorPage[BuildListResponse]]:
        """
        List Builds

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/builds",
            page=AsyncCursorPage[BuildListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    build_list_params.BuildListParams,
                ),
            ),
            model=BuildListResponse,
        )

    async def cancel(
        self,
        build_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BuildCancelResponse:
        """
        Cancel a pending or running build.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not build_id:
            raise ValueError(f"Expected a non-empty value for `build_id` but received {build_id!r}")
        return await self._post(
            f"/v5/builds/{build_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BuildCancelResponse,
        )

    async def logs(
        self,
        build_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[StreamChunk]:
        """
        Stream build logs via Server-Sent Events (SSE).

        Returns a streaming response with content-type text/event-stream. Each log line
        is sent as an SSE data event.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not build_id:
            raise ValueError(f"Expected a non-empty value for `build_id` but received {build_id!r}")
        return await self._get(
            f"/v5/builds/{build_id}/logs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StreamChunk,
            stream=True,
            stream_cls=AsyncStream[StreamChunk],
        )


class BuildResourceWithRawResponse:
    def __init__(self, build: BuildResource) -> None:
        self._build = build

        self.create = to_raw_response_wrapper(
            build.create,
        )
        self.retrieve = to_raw_response_wrapper(
            build.retrieve,
        )
        self.list = to_raw_response_wrapper(
            build.list,
        )
        self.cancel = to_raw_response_wrapper(
            build.cancel,
        )
        self.logs = to_raw_response_wrapper(
            build.logs,
        )


class AsyncBuildResourceWithRawResponse:
    def __init__(self, build: AsyncBuildResource) -> None:
        self._build = build

        self.create = async_to_raw_response_wrapper(
            build.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            build.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            build.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            build.cancel,
        )
        self.logs = async_to_raw_response_wrapper(
            build.logs,
        )


class BuildResourceWithStreamingResponse:
    def __init__(self, build: BuildResource) -> None:
        self._build = build

        self.create = to_streamed_response_wrapper(
            build.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            build.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            build.list,
        )
        self.cancel = to_streamed_response_wrapper(
            build.cancel,
        )
        self.logs = to_streamed_response_wrapper(
            build.logs,
        )


class AsyncBuildResourceWithStreamingResponse:
    def __init__(self, build: AsyncBuildResource) -> None:
        self._build = build

        self.create = async_to_streamed_response_wrapper(
            build.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            build.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            build.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            build.cancel,
        )
        self.logs = async_to_streamed_response_wrapper(
            build.logs,
        )



from unittest.mock import patch

import httpx
import pytest

from scale_gp_beta import SGPClient
from scale_gp_beta.lib.tracing.trace_queue_manager import TraceQueueManager


@pytest.fixture
def mock_transport() -> httpx.MockTransport:
    def handler(_request: httpx.Request):
        return httpx.Response(200, json={"ok": True})

    return httpx.MockTransport(handler)


@pytest.fixture
def real_sgp_client(mock_transport: httpx.MockTransport) -> SGPClient:
    """
    Creates a REAL SGPClient instance but with a mocked transport layer
    to prevent actual network calls.
    """
    return SGPClient(
        api_key="dummy_key",
        account_id="dummy_account_id",
        http_client=httpx.Client(transport=mock_transport),
    )


class TestRegisterClient:
    # Following 3 tests are to ensure that our monkey patch on SGPClient _prepare_request remains valid.
    # They should fail if Stainless changes some internals.
    def test_prepare_request_is_patched(self, real_sgp_client: SGPClient):
        """
        Verifies that our custom wrapper successfully replaces the '_prepare_request' method on the client instance.
        """
        original_method = real_sgp_client._prepare_request
        TraceQueueManager(worker_enabled=False, client=real_sgp_client)
        patched_method = real_sgp_client._prepare_request

        assert patched_method is not original_method
        assert patched_method.__name__ == "custom_prepare_request"

    def test_patch_calls_original_method(self, real_sgp_client: SGPClient):
        """
        Verifies that our custom wrapper correctly calls the original
        `_prepare_request` method, ensuring the chain of execution is not broken. Note that the original is default empty.
        But the user may have added something themselves.
        """
        manager = TraceQueueManager(worker_enabled=False)

        with patch.object(real_sgp_client, '_prepare_request', wraps=real_sgp_client._prepare_request) as mock_original:
            manager.register_client(real_sgp_client)

            real_sgp_client.spans.search()

            mock_original.assert_called_once()
            call_args = mock_original.call_args[0]
            assert len(call_args) == 1
            assert isinstance(call_args[0], httpx.Request)

    def test_patching_contract_against_real_client(self, real_sgp_client: SGPClient):
        """
        It ensures that our patching logic is compatible with the real client's
        method signature and request lifecycle. If Stainless changes the internal
        API, this test should fail.
        """
        TraceQueueManager(worker_enabled=False, client=real_sgp_client)

        try:
            # If `_prepare_request`'s signature changes, this line will raise an error.
            response = real_sgp_client.spans.search()
            assert response is not None
        except TypeError as e:
            pytest.fail(
                f"Contract broken! The signature of `_prepare_request` may have changed. Error: {e}"
            )
        except Exception as e:
            pytest.fail(f"An unexpected error occurred during the patched request: {e}")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ..types import (
    ApprovalStatus,
    AssessmentType,
    span_assessment_list_params,
    span_assessment_create_params,
    span_assessment_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncAPIListPage, AsyncAPIListPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.approval_status import ApprovalStatus
from ..types.assessment_type import AssessmentType
from ..types.span_assessment import SpanAssessment
from ..types.span_assessment_delete_response import SpanAssessmentDeleteResponse

__all__ = ["SpanAssessmentsResource", "AsyncSpanAssessmentsResource"]


class SpanAssessmentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpanAssessmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return SpanAssessmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpanAssessmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return SpanAssessmentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        assessment_type: AssessmentType,
        span_id: str,
        trace_id: str,
        approval: ApprovalStatus | Omit = omit,
        comment: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        overwrite: Dict[str, object] | Omit = omit,
        rating: int | Omit = omit,
        rubric: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessment:
        """
        Create new assessment for a span (comment, rating, approval, rubric, overwrite,
        or metadata)

        Args:
          assessment_type: Type of assessment

          span_id: The ID of the span this assessment is attached to

          trace_id: The ID of the trace this assessment is attached to

          approval: Approval status (approved/rejected)

          comment: Raw text feedback

          metadata: Arbitrary JSON object for additional data

          overwrite: User corrections to span output

          rating: Numerical rating (1-5)

          rubric: Rule key-value pairs for rubric evaluation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v5/span-assessments",
            body=maybe_transform(
                {
                    "assessment_type": assessment_type,
                    "span_id": span_id,
                    "trace_id": trace_id,
                    "approval": approval,
                    "comment": comment,
                    "metadata": metadata,
                    "overwrite": overwrite,
                    "rating": rating,
                    "rubric": rubric,
                },
                span_assessment_create_params.SpanAssessmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessment,
        )

    def retrieve(
        self,
        span_assessment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessment:
        """
        Get an assessment by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_assessment_id:
            raise ValueError(f"Expected a non-empty value for `span_assessment_id` but received {span_assessment_id!r}")
        return self._get(
            f"/v5/span-assessments/{span_assessment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessment,
        )

    def update(
        self,
        span_assessment_id: str,
        *,
        approval: ApprovalStatus | Omit = omit,
        assessment_type: AssessmentType | Omit = omit,
        comment: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        overwrite: Dict[str, object] | Omit = omit,
        rating: int | Omit = omit,
        rubric: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessment:
        """
        Update existing assessment (only by the original creator)

        Args:
          approval: Approval status (approved/rejected)

          assessment_type: Type of assessment

          comment: Raw text feedback

          metadata: Arbitrary JSON object for additional data

          overwrite: User corrections to span output

          rating: Numerical rating (1-5)

          rubric: Rule key-value pairs for rubric evaluation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_assessment_id:
            raise ValueError(f"Expected a non-empty value for `span_assessment_id` but received {span_assessment_id!r}")
        return self._patch(
            f"/v5/span-assessments/{span_assessment_id}",
            body=maybe_transform(
                {
                    "approval": approval,
                    "assessment_type": assessment_type,
                    "comment": comment,
                    "metadata": metadata,
                    "overwrite": overwrite,
                    "rating": rating,
                    "rubric": rubric,
                },
                span_assessment_update_params.SpanAssessmentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessment,
        )

    def list(
        self,
        *,
        assessment_type: AssessmentType | Omit = omit,
        span_id: str | Omit = omit,
        trace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncAPIListPage[SpanAssessment]:
        """
        Get all assessments for a specific span or trace, optionally filtered by
        assessment type

        Args:
          assessment_type: Filter by assessment type

          span_id: Filter by span ID. Either span_id or trace_id must be provided as a query
              parameter.

          trace_id: Filter by trace ID. Either span_id or trace_id must be provided as a query
              parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/span-assessments",
            page=SyncAPIListPage[SpanAssessment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "assessment_type": assessment_type,
                        "span_id": span_id,
                        "trace_id": trace_id,
                    },
                    span_assessment_list_params.SpanAssessmentListParams,
                ),
            ),
            model=SpanAssessment,
        )

    def delete(
        self,
        span_assessment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessmentDeleteResponse:
        """
        Delete assessment

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_assessment_id:
            raise ValueError(f"Expected a non-empty value for `span_assessment_id` but received {span_assessment_id!r}")
        return self._delete(
            f"/v5/span-assessments/{span_assessment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessmentDeleteResponse,
        )


class AsyncSpanAssessmentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpanAssessmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncSpanAssessmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpanAssessmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncSpanAssessmentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        assessment_type: AssessmentType,
        span_id: str,
        trace_id: str,
        approval: ApprovalStatus | Omit = omit,
        comment: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        overwrite: Dict[str, object] | Omit = omit,
        rating: int | Omit = omit,
        rubric: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessment:
        """
        Create new assessment for a span (comment, rating, approval, rubric, overwrite,
        or metadata)

        Args:
          assessment_type: Type of assessment

          span_id: The ID of the span this assessment is attached to

          trace_id: The ID of the trace this assessment is attached to

          approval: Approval status (approved/rejected)

          comment: Raw text feedback

          metadata: Arbitrary JSON object for additional data

          overwrite: User corrections to span output

          rating: Numerical rating (1-5)

          rubric: Rule key-value pairs for rubric evaluation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v5/span-assessments",
            body=await async_maybe_transform(
                {
                    "assessment_type": assessment_type,
                    "span_id": span_id,
                    "trace_id": trace_id,
                    "approval": approval,
                    "comment": comment,
                    "metadata": metadata,
                    "overwrite": overwrite,
                    "rating": rating,
                    "rubric": rubric,
                },
                span_assessment_create_params.SpanAssessmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessment,
        )

    async def retrieve(
        self,
        span_assessment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessment:
        """
        Get an assessment by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_assessment_id:
            raise ValueError(f"Expected a non-empty value for `span_assessment_id` but received {span_assessment_id!r}")
        return await self._get(
            f"/v5/span-assessments/{span_assessment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessment,
        )

    async def update(
        self,
        span_assessment_id: str,
        *,
        approval: ApprovalStatus | Omit = omit,
        assessment_type: AssessmentType | Omit = omit,
        comment: str | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        overwrite: Dict[str, object] | Omit = omit,
        rating: int | Omit = omit,
        rubric: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessment:
        """
        Update existing assessment (only by the original creator)

        Args:
          approval: Approval status (approved/rejected)

          assessment_type: Type of assessment

          comment: Raw text feedback

          metadata: Arbitrary JSON object for additional data

          overwrite: User corrections to span output

          rating: Numerical rating (1-5)

          rubric: Rule key-value pairs for rubric evaluation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_assessment_id:
            raise ValueError(f"Expected a non-empty value for `span_assessment_id` but received {span_assessment_id!r}")
        return await self._patch(
            f"/v5/span-assessments/{span_assessment_id}",
            body=await async_maybe_transform(
                {
                    "approval": approval,
                    "assessment_type": assessment_type,
                    "comment": comment,
                    "metadata": metadata,
                    "overwrite": overwrite,
                    "rating": rating,
                    "rubric": rubric,
                },
                span_assessment_update_params.SpanAssessmentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessment,
        )

    def list(
        self,
        *,
        assessment_type: AssessmentType | Omit = omit,
        span_id: str | Omit = omit,
        trace_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SpanAssessment, AsyncAPIListPage[SpanAssessment]]:
        """
        Get all assessments for a specific span or trace, optionally filtered by
        assessment type

        Args:
          assessment_type: Filter by assessment type

          span_id: Filter by span ID. Either span_id or trace_id must be provided as a query
              parameter.

          trace_id: Filter by trace ID. Either span_id or trace_id must be provided as a query
              parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/span-assessments",
            page=AsyncAPIListPage[SpanAssessment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "assessment_type": assessment_type,
                        "span_id": span_id,
                        "trace_id": trace_id,
                    },
                    span_assessment_list_params.SpanAssessmentListParams,
                ),
            ),
            model=SpanAssessment,
        )

    async def delete(
        self,
        span_assessment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanAssessmentDeleteResponse:
        """
        Delete assessment

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_assessment_id:
            raise ValueError(f"Expected a non-empty value for `span_assessment_id` but received {span_assessment_id!r}")
        return await self._delete(
            f"/v5/span-assessments/{span_assessment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanAssessmentDeleteResponse,
        )


class SpanAssessmentsResourceWithRawResponse:
    def __init__(self, span_assessments: SpanAssessmentsResource) -> None:
        self._span_assessments = span_assessments

        self.create = to_raw_response_wrapper(
            span_assessments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            span_assessments.retrieve,
        )
        self.update = to_raw_response_wrapper(
            span_assessments.update,
        )
        self.list = to_raw_response_wrapper(
            span_assessments.list,
        )
        self.delete = to_raw_response_wrapper(
            span_assessments.delete,
        )


class AsyncSpanAssessmentsResourceWithRawResponse:
    def __init__(self, span_assessments: AsyncSpanAssessmentsResource) -> None:
        self._span_assessments = span_assessments

        self.create = async_to_raw_response_wrapper(
            span_assessments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            span_assessments.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            span_assessments.update,
        )
        self.list = async_to_raw_response_wrapper(
            span_assessments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            span_assessments.delete,
        )


class SpanAssessmentsResourceWithStreamingResponse:
    def __init__(self, span_assessments: SpanAssessmentsResource) -> None:
        self._span_assessments = span_assessments

        self.create = to_streamed_response_wrapper(
            span_assessments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            span_assessments.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            span_assessments.update,
        )
        self.list = to_streamed_response_wrapper(
            span_assessments.list,
        )
        self.delete = to_streamed_response_wrapper(
            span_assessments.delete,
        )


class AsyncSpanAssessmentsResourceWithStreamingResponse:
    def __init__(self, span_assessments: AsyncSpanAssessmentsResource) -> None:
        self._span_assessments = span_assessments

        self.create = async_to_streamed_response_wrapper(
            span_assessments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            span_assessments.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            span_assessments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            span_assessments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            span_assessments.delete,
        )
